%% HOTDECK_IMPUTATION : HotDeck imputation method for missing values
%
% This nonparametric imputation method involves a search for complete cases that have
% the same values on the observed variables as the case with missing values, and then imputes
% the observed values of the k neighbors
%
%
% ARGUMENTS   : X    -> input data with missing values (N x m)
%               W    -> Indication matrix of where the missing values are to be found (N x m)
%                           0 no missing value
%                           1 missing value
%               k    -> number of imputations (default k=5)
%
% RETURN VALUE: Y    -> data with imputate values (N x m x k)
%
% DEPENDENCIES: neighbors.m               
%

function Y=HotDeck_imputation(X,W,k)


if nargin<3
   k=5;
end;

[N,m]=size(X);

Y=zeros(N,m,k);
sw=sum(W')';
for i=1:N,
    for h=1:k,
       Y(i,:,h) = X(i,:);
    end;
    %THERE ARE MISSINGS
    if sw(i) > 0,

       %SEARCH FOR NEIGHBORS
       if i == 1,
          Xa = X(2:N,:);
          Wa = W(2:N,:);
       elseif i == N,
          Xa = X(1:N-1,:);
          Wa = W(1:N-1,:);
       else
          Xa =[X(1:i-1,:); X(i+1:N,:)];
          Wa =[W(1:i-1,:); W(i+1:N,:)];
       end;
       Xv = neighbors(X(i,:),W(i,:),Xa,Wa,k);

       %RANDOMIZE NEIGHBORS
       [dummy, J]=sort(rand(k,1)); %J is a random permutation vector
       Xv = Xv(J,:);

       %to imputate
       for j=1:m,
           if W(i,j) == 1,
              for h=1:k,
                  Y(i,j,h)=Xv(h,j);
                  Xv(h,j);
              end;
           end;

       end;
     
    end;

end;
