%% EAP_CONTINUOUS_OBLI_CORRECTOR: Compute the EAP for continuous data (modified version)
%
% ARGUMENTS   : X    raw matrix.
%               P    Loading matrix.
%               PHI  Correlation matrix
%
% RETURN VALUE: th    factor scores.
%               th_li factor scores inferior limit.
%               th_ls factor scores upper limit.
%               se    standard errors
%               reli  reliability of factor scores
%
% MODIFICATION 20/2/2017: WE WILL USE THE MEAN, STANDARD DEVIATION AND
% CORRELATION MATRIX FROM THE ORIGINAL SAMPLE (data.Xa or
% data.X_sensemissa).
%
%       Xo -> Original data
%

function [th,th_li,th_ls,se,reli] = eap_continuous_obli_corrector(X,P,PHI,Matrix)

   [n,m] = size(X);
   [m,r]=size(P);
   
   data=getappdata(0,'data');
   
   if data.missings_on==1 %if the original data contains missings
       Xo=data.X_sensemissa; %original data
   else
       Xo=data.Xa; %original data
   end;
   
   M = mean(Xo); Sx = std(Xo);
   DIF = X - (ones(n,1) * M);
   Z = DIF ./ (ones(n,1) * Sx);

   switch Matrix
       case {1,2}
           R=corrcoef(Xo);
       case 3
           R=data.Rp;
   end
   

   th = (PHI*P'*R^(-1)*Z')';

   U = diag(diag(R - P*PHI*P'));

   VAR_E = (PHI^(-1) + P'*U^(-1)*P)^(-1);
   se = sqrt(diag(diag(VAR_E)));

   z90= 1.64485362695147;
  
% 90% approximate confidence interval
  th_li= zeros(n,r);
  th_ls= zeros(n,r);
  for i=1:n,
       tmp=[];
       for j=1:r,
           th_li(i,j) = th(i,j) - z90*se(j,j);
           th_ls(i,j) = th(i,j) + z90*se(j,j);
       end;
  end;

  reli=diag(eye - VAR_E);
   
return;
