%% IMPORT_DATA : Psychological Test Toolbox window for importing the data 
%
% This program is designed for computing factor analysis controlling two
% response bias: Social Desirability and Acquiescence.
%
% For detailed info about the program, please go to:
% http://psico.fcep.urv.cat/utilitats/PsychologicalTestToolbox
%
% Licensing:
%   This code is distributed under the GNU General Public License version
%   3.
%
% Authors:
%   David Navarro-Gonzalez
%   Andreu Vigil-Colet
%   Pere Joan Ferrando
%   Urbano Lorenzo-Seva
%
% Universitat Rovira i Virgili
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function varargout = import_data(varargin)

if nargin == 0  % LAUNCH GUI

    %load data
    data=getappdata(0,'data');

    %Set the letter for the menus
    data.font = ChooseFont('Helvetica','Arial','MS Sans Serif');

    finestra.N=860;
    finestra.M=380;
    pantalla=get(0,'ScreenSize');
    finestra.coordenades=[(pantalla(3)/2)-(finestra.N/2)  (pantalla(4)/2)-(finestra.M/2) finestra.N finestra.M];

    %Crear figura
    handles.figread = figure('Units','Pixels',...
        'color' , [0.941 0.941 0.941],...
        'Position',finestra.coordenades);
    tmp=get(0,'FactoryUIControlBackgroundColor');
    set(handles.figread, 'MenuBar','none','Name','Psychological Test Toolbox: Read Data','Toolbar','none','NumberTitle','off','resize','off');

    %PANEL DEL TAMANY DE LA MATRIU
    handles.panel_size=uipanel(handles.figread,'Units','pixels',...
        'position',[10 270 420 100],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Title','Size of data matrix',...
        'BackgroundColor', [.941 .941 .941]);

    %TEXT NOMBRE DE PARTICIPANTS
    handles.text1= uicontrol(handles.panel_size,'Style','text',...
        'Units','pixels',...
        'position',[10 50 200 20],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'ForegroundColor',[.2 .2 .2],...
        'HorizontalAlignment','left',...
        'FontWeight','normal',...
        'BackgroundColor', [.941 .941 .941],...
        'String','Number of participants:');

    buff='';
    %QUADRE EDITABLE DEL NOMBRE DE PARTICIPANTS
    handles.numberofpar= uicontrol(handles.panel_size,'Style','edit',...
        'Units','pixels',...
        'position',[250 50 150 25],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'TooltipString','Number of participants',...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','right',...
        'BackgroundColor', [1 1 1],...
        'String',data.N);

    %TEXT NOMBRE DE VARIABLES
    handles.text2= uicontrol(handles.panel_size,'Style','text',...
        'Units','pixels',...
        'position',[10 10 200 20],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'ForegroundColor',[.2 .2 .2],...
        'HorizontalAlignment','left',...
        'FontWeight','normal',...
        'BackgroundColor', [.941 .941 .941],...
        'String','Number of variables:');

    %QUADRE EDITABLE DEL NOMBRE DE VARIABLES
    handles.numberofvar= uicontrol(handles.panel_size,'Style','edit',...
        'Units','pixels',...
        'position',[250 10 150 25],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'TooltipString','Number of variables',...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','right',...
        'BackgroundColor', [1 1 1],...
        'String',data.n);
    
    %PANEL DELS MISSINGS
    handles.panel_missings=uipanel(handles.figread,'Units','pixels',...
        'position',[440 270 410 100],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Title','Missing values',...
        'BackgroundColor', [.941 .941 .941]);
    
    %CHECKBOX MISSINGS
    handles.check_missings = uicontrol(handles.panel_missings,...
        'style','checkbox',...
        'string','The data file contains missing values',...
        'fontname',char(data.font),...
        'FontSize',13,...
        'ForegroundColor',[0 0 0],...
        'HorizontalAlignment','left',...
        'value',0,...
        'BackgroundColor', [.941 .941 .941],...
        'position',[10 50 400 20]);
    
    %TEXT DEFINIR VALOR DELS MISSINGS
    handles.text3= uicontrol(handles.panel_missings,'Style','text',...
        'Units','pixels',...
        'position',[10 10 200 20],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'ForegroundColor',[.2 .2 .2],...
        'HorizontalAlignment','left',...
        'FontWeight','normal',...
        'visible','off',...
        'BackgroundColor', [.941 .941 .941],...
        'String','Defined missing value:');

    %QUADRE EDITABLE DEL VALOR DELS MISSING
    handles.missingvalue= uicontrol(handles.panel_missings,'Style','edit',...
        'Units','pixels',...
        'position',[250 10 150 25],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'TooltipString','Define wich value indicate the presence of a missing value',...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'Visible','off',...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','right',...
        'BackgroundColor', [1 1 1],...
        'String',data.missings_value);
    
    if isempty(data.missings_value)==0
        set(handles.check_missings,'visible','on','value',1);
        set(handles.text3,'visible','on');
        set(handles.missingvalue,'visible','on');
    end

    %PANEL DEL FITXER
    handles.panel_file=uipanel(handles.figread,'Units','pixels',...
        'position',[10 60 840 200],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Title','File name and options',...
        'BackgroundColor', [.941 .941 .941]);

    %Radio button: participants scores
    handles.radio_scores = uicontrol(handles.panel_file,...
        'style','radiobutton',...
        'string','Participant''s scores:',...
        'fontname',char(data.font),...
        'FontSize',13,...
        'ForegroundColor',[0 0 0],...
        'HorizontalAlignment','left',...
        'value',1,...
        'BackgroundColor', [.941 .941 .941],...
        'position',[10 145 200 20]);

    
    %QUADRE EDITABLE DE LES PUNTUACIONS DELS SUBJECTES
    handles.scores= uicontrol(handles.panel_file,'Style','edit',...
        'Units','pixels',...
        'position',[280 145 430 25],...
        'FontSize',13,...
        'fontname',char(data.font),...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','left',...
        'Enable','off',...
        'BackgroundColor', [1 1 1],...
        'String',data.path_X);

    %PUSH BUTTON DEL BROWSE DE PUNTUACIONS
    handles.browse1= uicontrol(handles.panel_file,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[728 142 100 30],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','Browse');

    %Radio button: variance/covariance matrix
    handles.radio_variance = uicontrol(handles.panel_file,...
        'style','radiobutton',...
        'string','Variance/covariance matrix:',...
        'fontname',char(data.font),...
        'FontSize',13,...
        'ForegroundColor',[0 0 0],...
        'HorizontalAlignment','left',...
        'value',0,...
        'BackgroundColor', [.941 .941 .941],...
        'position',[10 100 300 20]);

    %QUADRE EDITABLE DE LA MATRIU DE CORRELACIONS
    handles.variance= uicontrol(handles.panel_file,'Style','edit',...
        'Units','pixels',...
        'position',[280 100 430 25],...
        'FontSize',12,...
        'fontname',char(data.font),...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','left',...
        'Enable','off',...
        'BackgroundColor', [1 1 1],...
        'String',data.path_S);

    %PUSH BUTTON DEL BROWSE DE MATRIU DE CORRELACIONS
    handles.browse2= uicontrol(handles.panel_file,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[728 97 100 30],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Enable','off',...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','Browse');
    
    if isempty(data.path_X) && isempty(data.path_S)==0
        set(handles.radio_scores,'value',1);
        set(handles.radio_variance,'value',0);
        set(handles.browse1,'enable','on');
    end
    
    %CHECKBOX LABELS
    handles.check_labels = uicontrol(handles.panel_file,...
        'style','checkbox',...
        'string','Variable Labels:',...
        'fontname',char(data.font),...
        'FontSize',13,...
        'ForegroundColor',[0 0 0],...
        'HorizontalAlignment','left',...
        'value',0,...
        'BackgroundColor', [.941 .941 .941],...
        'position',[10 55 300 20]);

    %QUADRE EDITABLE DE LABELS
    handles.labels= uicontrol(handles.panel_file,'Style','edit',...
        'Units','pixels',...
        'position',[280 55 430 25],...
        'FontSize',12,...
        'fontname',char(data.font),...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','left',...
        'Enable','off',...
        'BackgroundColor', [1 1 1],...
        'String',data.path_labels);    

    %PUSH BUTTON DEL BROWSE DE LABELS
    handles.browse3= uicontrol(handles.panel_file,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[728 52 100 30],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Enable','off',...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','Browse');
    
    if isempty(data.path_labels)==0
        set(handles.check_labels,'value',1);
        set(handles.browse3,'enable','on');
    end
    
    %CHECKBOX TARGET MATRIX
    handles.check_target = uicontrol(handles.panel_file,...
        'style','checkbox',...
        'string','Rotation Target Matrix:',...
        'fontname',char(data.font),...
        'FontSize',13,...
        'ForegroundColor',[0 0 0],...
        'HorizontalAlignment','left',...
        'value',0,...
        'BackgroundColor', [.941 .941 .941],...
        'position',[10 10 300 20]);

    %QUADRE EDITABLE DE LA TARGET ROTATION MATRIX
    handles.target= uicontrol(handles.panel_file,'Style','edit',...
        'Units','pixels',...
        'position',[280 10 430 25],...
        'FontSize',12,...
        'fontname',char(data.font),...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'HorizontalAlignment','left',...
        'Enable','off',...
        'BackgroundColor', [1 1 1],...
        'String',data.path_target);
    
    %PUSH BUTTON DEL BROWSE DEL TARGET
    handles.browse4= uicontrol(handles.panel_file,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[728 7 100 30],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'Enable','off',...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','Browse');

    if isempty(data.path_target)==0
        set(handles.check_target,'value',1);
        set(handles.browse4,'enable','on');
    end

    %CANCEL button
    handles.cancel= uicontrol(handles.figread,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[585 10 120 35],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','Cancel');

    %OK button
    handles.ok= uicontrol(handles.figread,'Style','Pushbutton',...
        'Units','pixels',...
        'position',[725 10 120 35],...
        'Fontname',char(data.font),...
        'Fontsize',13,...
        'ForegroundColor',[.2 .2 .2],...
        'BackgroundColor',[.911 .911 .911],...
        'String','OK');

    %CRIDEM ELS CALLBACKS

    set(handles.check_missings,...
        'Callback',{@Check_missings_Callback handles});
    
    set(handles.radio_scores,...
        'Callback',{@Matrix_Callback,handles,1});
    set(handles.radio_variance,...
        'Callback',{@Matrix_Callback,handles,2});
    set(handles.check_labels,...
        'Callback',{@Labels_Callback, handles});
    set(handles.check_target,...
        'Callback',{@Target_Callback, handles});

    set(handles.browse1,...
        'Callback',{@Browse1_Callback, handles});
    set(handles.browse2,...
        'Callback',{@Browse2_Callback, handles});
    set(handles.browse3,...
        'Callback',{@Browse3_Callback, handles});
    set(handles.browse4,...
        'Callback',{@Browse4_Callback, handles});

    set(handles.cancel,...
        'Callback',{@Cancel_Callback});
    set(handles.ok,...
        'Callback',{@Ok_Callback, handles});

    if nargout > 0
            varargout{1} = handles.figread;
    end


elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end

end

function Check_missings_Callback (hObject,eventdata,handles)
    
    check=get(handles.check_missings,'value');
    if check==1
        set(handles.missingvalue,'visible','on');
        set(handles.text3,'visible','on');
    else
        set(handles.missingvalue,'visible','off');
        set(handles.text3,'visible','off');
    end
    
return;

function varargout = Matrix_Callback (gcf,event_data,handles, radio_value,varargin)
    
    data=getappdata(0,'data');
    data.path_X=[];
    data.path_S=[];

    switch radio_value       
        case 1
            set(handles.radio_scores,'value',1);
            set(handles.radio_variance,'value',0);
            set(handles.browse1,'enable','on');
            set(handles.browse2,'enable','off');
            set(handles.panel_missings,'visible','on');
            set(handles.check_missings,'value',0);
            buff=[];
            set(handles.missingvalue,'string',buff);
            set(handles.variance,'string',buff);

        case 2
            set(handles.radio_scores,'value',0);
            set(handles.radio_variance,'value',1);
            set(handles.browse1,'enable','off');
            set(handles.browse2,'enable','on');
            set(handles.panel_missings,'visible','off');
            set(handles.check_missings,'value',0);
            buff=[];
            set(handles.missingvalue,'string',buff);
            set(handles.scores,'string',buff);
    end
    
    setappdata(0,'data',data);
            
return;

function varargout = Labels_Callback (gcf,event_data,handles,varargin)

    val=get(handles.check_labels,'value');
    data=getappdata(0,'data');
    data.path_labels=[];

    if val
        set(handles.browse3,'enable','on');
    else
        set(handles.browse3,'enable','off');
        set(handles.labels,'string','');
    end
    
    setappdata(0,'data',data);

return;

function varargout = Target_Callback (gcf,event_data,handles,varargin)

    val2=get(handles.check_target,'value');
    data=getappdata(0,'data');
    data.path_target=[];

    if val2
%         set(handles.target,'enable','on');
        set(handles.browse4,'enable','on');
    else
%         set(handles.target,'enable','off');
        set(handles.browse4,'enable','off');
        set(handles.target,'string','');
    end
    
    setappdata(0,'data',data);
    
return;

function varargout = Cancel_Callback (gcf,event_data,handles,varargin)

    %load data
    data=getappdata(0,'data');

    delete(gcbf);

return;

function Browse1_Callback (hObject,eventdata,handles)

    [filename1, pathname1,filterindex] = uigetfile({'*.dat';'*.txt';'*.mat';'*.xls';'*.xlsx';'*.*'},'Psychological Test Toolbox');
    
    if isequal(filename1,0)||isequal(pathname1,0)
        return;
    else
        a=uiimport([pathname1 filename1]);
        if isempty(a)
            return;
        end
        %load data
        data=getappdata(0,'data');
        
       set(handles.scores,'string',[pathname1, filename1]);
       set(handles.variance,'string','');
        
        [~,m]=size(filename1);
        campana=0;variable='';
        for i=1:m
            if filename1(i) == '.'
                campana = 1;
            end
            if campana == 1
                break
            else
                if filename1(i) == ' '
                else
                    variable = [variable filename1(i)];
                end
            end
        end

        status = xlsfinfo([pathname1 filename1]);
        trf=strcmp(status,'Microsoft Excel Spreadsheet');
        names=fieldnames(a);
        
        %check if there is a text field
        if isfield(a, "textdata")
            tmp={};
            dlg{1}='The program has detected a text variable that can not be analyzed.';
            dlg{2}='Please, provide a file containing only numerical variables.';
            errordlg(dlg,'Psychological Test Toolbox');
            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
            set(handles.radio_variance,'ForegroundColor',[0 0 0]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);
            set(handles.scores,'string',tmp);
            set(handles.check_labels,'value',0);
            X1=[];
            return;
        end
        
        [f1,~]=size(names); %comprovem si ha generat diverses estructures
        if f1>1 %si ha generat moltes estructures, ha carregat vectors amb cada item
            if f1>1 && f1<4 %si ha carregat les etiquetes, suggerim si les vol utilitzar
                X1=a.data;
            else
                X1=[];
                for i=1:f1
                    eval(['X1 = [X1 a.' char(names(i)) ']']);
                end
            end

            if trf==1
                ButtonName = questdlg('The excel spreadsheet uploaded contains column headers. Do you want to use them as labels?','Psychological Test Toolbox','Yes','No','No');
            else
                ButtonName = questdlg('The file uploaded contains column headers. Do you want to use them as labels?','Psychological Test Toolbox','Yes','No','No');
            end

            switch ButtonName
            case 'No'
                % take no action
            case 'Yes'
                if f1>1 && f1<4
                    data.labels=a.colheaders';
                else
                    data.labels=names;
                end
                set(handles.check_labels,'value',1);
                set(handles.labels,'string',[pathname1 filename1]);
                set(handles.browse3,'enable','on');
            end
        else
            if trf==1
                X1=a.data;
            else
                if (isfield(a, "data"))
                    %The program has failed to assing the name of the file
                    %in to the structure and has called data
                    X1 = a.data;
                else
                    if isfield(a, variable)
                        eval(['X1 = a.' variable ';']);
                    else
                        % Probably a MATLAB file, where the name of the
                        % file does not imply that the structure field has
                        % the same name
                        if f1 == 1 %check if only one structure field has
                                   %been imported, otherwise return
                            char_name = char(names);
                        	eval(['X1 = a.' char_name ';']);
                            
                        else
                            %impossible to import
                            tmp={};
                            dlg{1}='The program has detected multiple variable in the mat file.';
                            dlg{2}='Please, provide select only one variable containing the data.';
                            errordlg(dlg,'Psychological Test Toolbox');
                            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
                            set(handles.radio_variance,'ForegroundColor',[0 0 0]);
                            set(handles.check_labels,'ForegroundColor',[0 0 0]);
                            set(handles.check_target,'ForegroundColor',[0 0 0]);
                            set(handles.scores,'string',tmp);
                            set(handles.check_labels,'value',0);
                            X1=[];
                            return;
                        end
                    end
                    
                end
            end
        end

        %Check if the matrix contains some missings values
        tmp={};
        [row,col]=find(isnan(X1));
        if isempty(row)==0
            [~,g2]=size(X1);
            %Check if the data missings are correctly identified by the
            %uiimport function. If all the missing values are situated at
            %the end of the matrix, the function had not indentified
            %correctly the missing values, and we have to do it manually
            
            if min(col)>g2-(0.2*g2) %problem
                
                prompt{1} = 'There was a problem trying to import your data. It was ';
                prompt{2} = 'probably caused by the missing values found in your data, ';
                prompt{3} = 'causing that the program was unable to indetify those properly.';
                prompt{4} = 'The program will try to import data with another function. ';
                prompt{5} = 'Do you want to continue?';
                answ=questdlg(prompt,'Psychological Test Toolbox');
                switch answ
                    case 'Yes'
                        prompt = 'Please, select the separator used in your data:';
                        dlg_title = 'Psychological Test Toolbox';
                        num_lines = 1;
                        defaultans = {' '};
%                         answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
                        answer = inputdlg(prompt,dlg_title);
                        if isempty(answer)
                            set(handles.labels,'string',tmp);
                            set(handles.scores,'string',tmp);
                            set(handles.check_labels,'value',0);
                            X1=[];
                            return;
                        end
                    case 'No'
                        set(handles.labels,'string',tmp);
                        set(handles.scores,'string',tmp);
                        set(handles.check_labels,'value',0);
                        X1=[];
                        return;
                    case 'Cancel'
                        set(handles.labels,'string',tmp);
                        set(handles.scores,'string',tmp);
                        set(handles.check_labels,'value',0);
                        X1=[];
                        return;
                        
                end
                
                X1=dlmread([pathname1 filename1],char(answer));
                [~,h2]=size(X1);
                if h2~=g2 %problem, it is not the appropiate separator
                    dlg{1}='The program was unable to import your data. Please, replace all missing values ';
                    dlg{2}='for one specific value (for example 999) and then reupload the file';
                    errordlg(dlg,'Psychological Test Toolbox');
                    set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
                    set(handles.radio_variance,'ForegroundColor',[0 0 0]);
                    set(handles.check_labels,'ForegroundColor',[0 0 0]);
                    set(handles.check_target,'ForegroundColor',[0 0 0]);
                    set(handles.scores,'string',tmp);
                    set(handles.labels,'string',tmp)
                    set(handles.check_labels,'value',0);
                    X1=[];
                    return;
                    
                else
                    %Everything seems OK! Just one more comprobation:
                    %dlmread treats NaN values as 0. We will replace those
                    %zero values for 999 in order to consider those as
                    %missing values. The only remaining problem is if in
                    %the original data, 0 is a possible value. We will
                    %check that
                    [j1,~]=size(col); %number of NaN values found on the first import (uiimport)
                    
                    buff=find(X1==0);
                    [numberofzeros,~]=size(buff);
                    if j1~=numberofzeros %problem, there are more zero values than missing values, we can't do anything more
                        dlg{1}='The program was unable to import your data. Please, replace all missing values ';
                        dlg{2}='for one specific value (for example 999) and then reupload the file';
                        errordlg(dlg,'Psychological Test Toolbox');
                        set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
                        set(handles.radio_variance,'ForegroundColor',[0 0 0]);
                        set(handles.check_labels,'ForegroundColor',[0 0 0]);
                        set(handles.check_target,'ForegroundColor',[0 0 0]);
                        set(handles.scores,'string',tmp);
                        set(handles.labels,'string',tmp)
                        set(handles.check_labels,'value',0);
                        X1=[];
                        return;
                        
                    else %OK
                        dlg{1}='Apparently there are some missing values in your data. ';
                        dlg{2}='If this info is correct, the program will replace this values for ';
                        dlg{3}='an specific value in order to process them as missing values.';
                        dlg{4}='Do you want to continue?';
                        answ=questdlg(dlg,'Psychological Test Toolbox');

                        switch answ
                            case 'Yes'
%                                 X1(isnan(X1))=999;
                                X1(X1==0)=999;
                                set(handles.check_missings,'value',1);
                                set(handles.missingvalue,'visible','on','string',999);
                                set(handles.text3,'visible','on');
                                msgbox('All missing values had been replaced by number 999','Psychological Test Toolbox');

                            case 'No'
                                set(handles.labels,'string',tmp);
                                set(handles.scores,'string',tmp);
                                set(handles.check_labels,'value',0);
                                X1=[];
                                return;

                            case 'Cancel'
                                set(handles.labels,'string',tmp);
                                set(handles.scores,'string',tmp);
                                set(handles.check_labels,'value',0);
                                X1=[];
                                return;
                        end
                    end
                end

            else
            
                dlg{1}='Apparently there are some missing values in your data. ';
                dlg{2}='If this info is correct, the program will replace this values for ';
                dlg{3}='an specific value in order to process them as missing value.';
                dlg{4}='Do you want to continue?';
                answ=questdlg(dlg,'Psychological Test Toolbox');

                switch answ
                    case 'Yes'
                        X1(isnan(X1))=999;
                        set(handles.check_missings,'value',1);
                        set(handles.missingvalue,'visible','on','string',999);
                        set(handles.text3,'visible','on');
                        msgbox('All missing values had been replaced by number 999','Psychological Test Toolbox');

                    case 'No'
                        set(handles.labels,'string',tmp);
                        set(handles.scores,'string',tmp);
                        set(handles.check_labels,'value',0);
                        X1=[];
                        return;

                    case 'Cancel'
                        set(handles.labels,'string',tmp);
                        set(handles.scores,'string',tmp);
                        set(handles.check_labels,'value',0);
                        X1=[];
                        return;
                end
            end
        end
        
       [f1,f2]=size(X1);
       
       % Check if it is empty (in mat files it is possible)
       if isempty(X1)
            set(handles.labels,'string',tmp);
            set(handles.scores,'string',tmp);
            set(handles.check_labels,'value',0);
            X1=[];
            return;
           
       end
       
       % Check if it's a matrix
        if f2 < 2 || f1 < 2
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
            set(handles.radio_variance,'ForegroundColor',[0 0 0]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);
            set(handles.labels,'string',tmp);
            set(handles.scores,'string',tmp);
            set(handles.check_labels,'value',0);
            X1=[];
            return;
        end
        
       set(handles.radio_scores,'ForegroundColor',[0 0 0]);
       set(handles.numberofpar,'string',f1);
       set(handles.numberofvar,'string',f2);
    end
    
    
    data.X=X1;
    data.name=filename1;
    data.path=[pathname1 filename1];
    data.path_X=[pathname1 filename1];
    data.path_S=[];
    setappdata(0,'data',data);

    
return;

function Browse2_Callback (hObject,eventdata,handles)

    [filename1, pathname1] = uigetfile({'*.dat';'*.txt';'*.mat';'*.xls';'*.xlsx';'*.*'},'Psychological Test Toolbox');
    
    if isequal(filename1,0)||isequal(pathname1,0)
        return;
    else
        a=uiimport([pathname1 filename1]);
        if isempty(a)
            return;
        end
       set(handles.variance,'string',[pathname1, filename1]);
       set(handles.scores,'string','');
         
        [~,m]=size(filename1);
        campana=0;variable='';
        for i=1:m
            if filename1(i) == '.'
                campana = 1;
            end
            if campana == 1
                break
            else
                if filename1(i) == ' '
                else
                    variable = [variable filename1(i)];
                end
            end
        end
        
        %load data
        data=getappdata(0,'data');
        
        status = xlsfinfo([pathname1 filename1]);
        trf=strcmp(status,'Microsoft Excel Spreadsheet');
        names=fieldnames(a);
        [f1,~]=size(names); %comprovem si ha generat diverses estructures
        if f1>1 %si ha generat moltes estructures, ha carregat vectors amb cada item
            if f1>1 && f1<4 %si ha carregat les etiquetes, suggerim si les vol utilitzar
                S1=a.data;
            else
                S1=[];
                for i=1:f1
                    eval(['S1 = [S1 a.' char(names(i)) ']']);
                end
            end

            if trf==1
                ButtonName = questdlg('The excel spreadsheet uploaded contains column headers. Do you want to use them as labels?','Psychological Test Toolbox','Yes','No','No');
            else
                ButtonName = questdlg('The file uploaded contains column headers. Do you want to use them as labels?','Psychological Test Toolbox','Yes','No','No');
            end

            switch ButtonName
            case 'No'
                % take no action
            case 'Yes'
                if f1>1 && f1<4
                    data.labels=a.colheaders';
                else
                    data.labels=names;
                end
                set(handles.check_labels,'value',1);
                set(handles.labels,'string',[pathname1 filename1]);
                set(handles.browse3,'enable','on');
            end 
        else
            if trf==1
                S1=a.data;
            else
            eval(['S1 = a.' variable ';']);
            end
        end

        [f1,f2]=size(S1);

        % Check if it's a matrix
        if f2 < 2 || f1 < 2
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            set(handles.radio_scores,'ForegroundColor',[0 0 0]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);
            return;
        end
        
        % Check if contains NaN
        [row, ~] = find(isnan(S1));
        tmp='';
        if isempty(row)
        else
            errordlg('The file contains some NaN values. When importing dispersion matrix, NaN values are not accepted.','Psychological Test Toolbox');
            set(handles.radio_scores,'ForegroundColor',[0 0 0]);
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);
            set(handles.labels,'string',tmp)
            set(handles.check_labels,'value',0);
            return;
        end
        
        if f2==f1 % dispersion matrix (square one)

            set(handles.numberofvar,'string',f2);
            set(handles.radio_variance,'ForegroundColor',[0 0 0]);
            set(handles.radio_scores,'ForegroundColor',[0 0 0]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);

        else
            errordlg('The variable must be a square matrix','Psychological Test Toolbox','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            set(handles.radio_scores,'ForegroundColor',[0 0 0]);
            set(handles.check_labels,'ForegroundColor',[0 0 0]);
            set(handles.check_target,'ForegroundColor',[0 0 0]);
            return;
        end        
    end
    
    
    data.S=S1;
    data.name=filename1;
    data.path=[pathname1 filename1];
    data.path_X=[];
    data.path_S=[pathname1 filename1];
    setappdata(0,'data',data);
    
return;

function Browse3_Callback (hObject,eventdata,handles)

    [filename1, pathname1] = uigetfile({'*.txt';'*.txt';'*.mat';'*.xls';'*.xlsx';'*.*'},'Psychological Test Toolbox');
    
    if isequal(filename1,0)||isequal(pathname1,0)
        return;
    else
        a=uiimport([pathname1 filename1]);
        if isempty(a)
            return;
        end
       set(handles.labels,'string',[pathname1, filename1]);
        
        [~,m]=size(filename1);
        campana=0;variable='';
        for i=1:m
            if filename1(i) == '.'
                campana = 1;
            end
            if campana == 1
                break
            else
                if filename1(i) == ' '
                else
                    variable = [variable filename1(i)];
                end
            end
        end

        eval(['X1 = a.' variable ';']);
    end
    
    %load data
    data=getappdata(0,'data');
    data.labels=X1;
    data.path_labels=[pathname1 filename1];
    setappdata(0,'data',data);

return;

function Browse4_Callback (hObject,eventdata,handles)

    [filename1, pathname1] = uigetfile({'*.dat';'*.txt';'*.mat';'*.xls';'*.xlsx';'*.*'},'Psychological Test Toolbox');
    
    if isequal(filename1,0)||isequal(pathname1,0)
        return;
    else
        a=uiimport([pathname1 filename1]);
         if isempty(a)
            return;
         end
       set(handles.target,'string',[pathname1, filename1]);
       
        [~,m]=size(filename1);
        campana=0;variable='';
        for i=1:m
            if filename1(i) == '.'
                campana = 1;
            end
            
            if campana == 1
                break
            else
                if filename1(i) == ' '
                else
                    variable = [variable filename1(i)];
                end           
            end
        end

        eval(['X1 = a.' variable ';']);
    end
    
    %load data
    data=getappdata(0,'data');
    data.H=X1;
    data.path_target=[pathname1 filename1];
    setappdata(0,'data',data);
    
return;

function varargout = Ok_Callback (gcbf,event_data,handles,varargin)

    evalin('base','clear');

    set(handles.numberofpar,'backgroundcolor',[1 1 1]);
    set(handles.numberofvar,'backgroundcolor',[1 1 1]);
    set(handles.radio_scores,'ForegroundColor',[0 0 0]);
    set(handles.radio_variance,'ForegroundColor',[0 0 0]);
    set(handles.check_labels,'ForegroundColor',[0 0 0]);
    set(handles.check_target,'ForegroundColor',[0 0 0]);
    set(handles.missingvalue,'backgroundcolor',[1 1 1]);

    val_N=get(handles.numberofpar,'string');
    val_n=get(handles.numberofvar,'string');

    val_scores=get(handles.radio_scores,'value');
    name_scores=get(handles.scores,'string');

    val_variance=get(handles.radio_variance,'value');
    name_variance=get(handles.variance,'string');

    val_labels=get(handles.check_labels,'value');
    name_labels=get(handles.labels,'string');

    val_target=get(handles.check_target,'value');
    name_target=get(handles.target,'string');
    
    val_miss=get(handles.check_missings,'value');
    value_miss=get(handles.missingvalue,'string');

    %CHECK IF NUMBER OF PARTICIPANTS AND ITEMS ARE NUMERIC ONES
    num1=isstrprop(val_N,'digit'); 
    find1=find(num1==0); 
    empt1=isempty(find1); 

    num2=isstrprop(val_n,'digit');
    find2=find(num2==0);
    empt2=isempty(find2);
    
    num3_1=isstrprop(value_miss,'digit');
    num3_2=isstrprop(value_miss,'punct'); %If it is a negative value
    find3_1=find(num3_1==0);
    empt3_1=isempty(find3_1);
    if empt3_1==0
        find3_2=find(num3_2==0);
        rev_empt3_2=isempty(find3_2);
    end
    
    numb_N=str2num(val_N);
    numb_n=str2num(val_n);

    if isempty(val_N)
        errordlg('Please, select the number of participants','Psychological Test Toolbox: Error','Error');
        set(handles.numberofpar,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif empt1==0
        errordlg('Please, insert a valid number of participants','Psychological Test Toolbox: Error','Error');
        set(handles.numberofpar,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif numb_N<1
        errordlg('Please, insert a positive number of participants','Psychological Test Toolbox: Error','Error');
        set(handles.numberofpar,'backgroundcolor',[.8 .2 .2]);
        return;
    end

    if isempty(val_n)
        errordlg('Please, select the number of variables','Psychological Test Toolbox: Error','Error');
        set(handles.numberofvar,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif empt2==0
        errordlg('Please, insert a valid number of variables','Psychological Test Toolbox: Error','Error');
        set(handles.numberofvar,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif numb_n<1
        errordlg('Please, insert a positive number of variables','Psychological Test Toolbox: Error','Error');
        set(handles.numberofpar,'backgroundcolor',[.8 .2 .2]);
        return;
    end

    if val_scores==1
        if isempty(name_scores)
            errordlg('Please, select the matrix that contains the items scores','Psychological Test Toolbox: Error','Error');
            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
            return;
        end
    end

    if val_variance==1
        if isempty(name_variance)
            errordlg('Please, select the matrix that contains the variance/covariance matrix','Psychological Test Toolbox: Error','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            return;
        end
    end

    if val_labels==1
        if isempty(name_labels)
            errordlg('Please, select the variables labels or dismark the checkbox','Psychological Test Toolbox: Error','Error');
            set(handles.check_labels,'ForegroundColor',[.8 .2 .2]);
            return;
        end
    end

    if val_target==1
        if isempty(name_target)
            errordlg('Please, select the rotation target matrix or dismark the checkbox','Psychological Test Toolbox: Error','Error');
            set(handles.check_target,'ForegroundColor',[.8 .2 .2]);
            return;
        end
    end
    
    if val_miss==1
        if isempty(value_miss) || (empt3_1==0 && rev_empt3_2~=0)
            errordlg('Please, insert a valid missing value or dismark the checkbox','Psychological Test Toolbox: Error','Error');
            set(handles.missingvalue,'backgroundcolor',[.8 .2 .2]);
            return;
        end
    end
            

    %load data
    data=getappdata(0,'data');
%     data.name=[];
    data.N=[];
    data.n=[];
    data.labelsa=[];
    data.labels_user_on=[];
    data.inclosos=[];
    data.exclosos=[];
    data.Matrix=[];

    
    data.N = str2num(val_N);
    data.n = str2num(val_n);


    % MISSINGS EXISTENCE
    
    checking=get(handles.check_missings,'value');
    
    if checking==1 % The data file contains missing values
        data.missings_on=1; 
        data.missings_value=str2num(get(handles.missingvalue,'string'));
    else
        data.missings_on=0;
        data.missings_value=[];
    end
    
    % RAW SCORES
    
    if val_scores==1    


%         X1=readfile(name_scores);   % Read the file
        X1=data.X;

        [f1,f2]=size(X1);

        %comprovem que es una MATRIU
        if f2 < 2 
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
            return;
        end
        if f1 < 2 
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_scores,'ForegroundColor',[.8 .2 .2]);
            return;
        end

        % Check if the number of items and participants are consistent with
        % the uploaded data
        if f1<data.N
            errordlg('The selected matrix seems to have less participants than you have indicated','Psychological Test Toolbox: Error','Error');
            set(handles.numberofpar,'backgroundcolor',[.8 .2 .2]);
            return;
        elseif f2<data.n
            errordlg('The selected matrix seems to have less variables than you have indicated','Psychological Test Toolbox: Error','Error');
            set(handles.numberofvar,'backgroundcolor',[.8 .2 .2]);
            return;
        end

        % Checking missings presence
        
        X1=X1(1:data.N,1:data.n); 
        
        if data.missings_on==1 % There are some missing values
            
            X_miss = X1 == data.missings_value ;
            
            data.missings_position=X_miss;

            max1=max(X_miss);
            max2=max(max1);

            if max2==0
                warndlg('The data file does not contain any specified missing value. Please, make sure that the value is the right one or check the data file.','Psychological Test Toolbox: Warning');
                set(handles.missingvalue,'backgroundcolor',[.8 .2 .2]);
                return;
            end
        else
                
        end
        
        data.X=X1;
        data.S =[]; 

    else % USER DEFINED MATRIX

%         S1 = readfile(name_variance);
        S1=data.S;

        [f1,f2]=size(S1);

        % Check if it's a matrix
        if f2 < 2 
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            return;
        end
        if f1 < 2 
            errordlg('The variable must be a matrix','Psychological Test Toolbox','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            return;
        end
        if f2==f1 %matriu
        else
            errordlg('The variable must be a square matrix','Psychological Test Toolbox','Error');
            set(handles.radio_variance,'ForegroundColor',[.8 .2 .2]);
            return;
        end

        % Check if the number of items and participants are consistent with
        % the uploaded data
        if f1<data.n || f2<data.n
            errordlg('The selected matrix seems to have less variables than you have indicated','Psychological Test Toolbox: Error','Error');
            set(handles.numberofvar,'backgroundcolor',[.8 .2 .2]);
            return;
        end
        data.S = S1(1:data.n,1:data.n);
        data.X = [];
    
    end

    %LABEL

    if val_labels==0
        data.labels=[]; 
        data.labels_user_on=0;
    else
        labels1=data.labels;
    
    % Depending how the labels are specified, importdata will give different output.
    % If the labels are well separated, the resulting vector is [nx1]. If
    % not, structure is generated with some components.
    
        data.labels_user_on=1;
        [f1,f2]=size(labels1); 

        if f1 < data.n % There are less labels than items
            
            %if the import window is invoked once the labels have been used
            %in the principal window, the structure will change from [nx1]
            %to [1xn]
            if f2 == data.n
                %OK
            else
                errordlg('The number of variables indicated is larger than the labels included in the label file. Maybe the labels are not properly sepparated.','Psychological Test Toolbox','Error');
                set(handles.check_labels,'ForegroundColor',[.8 .2 .2]);
                return;
            end
        end
        if f1 == data.n
            buff = labels1(1:data.n,1); 
            data.labels=buff';
        end
    end

    %TARGET
    % Check if the user has defined target matrix
    if val_target==0
        data.H=[];
    else
%         target1 = readfile(name_target);
        target1=data.H;

        [f1,~]=size(target1);
        % Check if the number of items are consistent with the target
        % matrix
        if f1<data.n 
            errordlg('The target matrix seems to have less variables than you have indicated','Psychological Test Toolbox: Error','Error');
            set(handles.numberofvar,'backgroundcolor',[.8 .2 .2]);
            return;
        end

        data.H = target1(1:data.n,:); 
    end

    data.Matrix=2;    
    
    %SAVE DATA 
    setappdata(0,'data',data);

    gcbf=handles.figread;
    delete(gcbf);

return;

function X = readfile(str2)

       try  
            str1 = {'load('''};
            str3 = {''');'};
            str4 = char(strcat(str1,str2,str3));
            eval(str4);

       catch
            errordlg(lasterr,'Psychological Test Toolbox: Error','error');
            return;
       end
       
    [~,m]=size(str2);
    campana=0;variable='';
    for i=m:-1:1
         if str2(i) == '/'
            break;
         end
        if str2(i) == '\'
            break;
        end
        if campana == 1
            variable = [variable str2(i)];
        end
        if str2(i) == '.'
            campana = 1;
        end
    end
    
    [~,m]=size(variable);
    variable_final='';
    for i=m:-1:1
        variable_final =[variable_final variable(i)];
    end
    
    str1=[ 'X = ' variable_final ';'];
    eval(str1);
    


return;

function name = readfile2(str2)

       try      
            str1 = {'load('''};
            str3 = {''');'};
            str4 = char(strcat(str1,str2,str3));
            eval(str4);

       catch
            errordlg(lasterr,'Psychological Test Toolbox: Error','error');
            return;
       end
       
    token = strtok(str2, '.');
    [~,m]=size(token);
    variable2='';
    for i=m:-1:1
        variable2=[variable2 token(i)];
    end
    
    token2 = strtok(variable2, '/');
    token3 = strtok(token2, '\');
    
    [~,m]=size(token3);
    variable_final='';
    for i=m:-1:1
        variable_final=[variable_final token3(i)];
    end
    
    name=variable_final;
   

return;
