%% MY_MINRES : MINRES extraction by BY Zegers & ten Berge (1983)
%
% ARGUMENTS   : C    -> direct, covariance or correlation matrix
%               q    -> number of components
%               conv -> convergence value
%               N    -> Number of subjects
%
% RETURN VALUE: A  -> Loadings
%               Re -> Residuals
%               Um -> Index of fit
%
% DEPENDENCIES: pcaR.m
%

function [A, Re, Um] = my_minres (C,q,conv,N)

%N = 1000;

 [B,A] = pcaR(C,q);
 [n,m] = size(A);
 Co = C - diag(diag(C));
 g = trace((Co - A*A' - diag(diag(A*A')))'*(Co - A*A' - diag(diag(A*A'))));
 gold = g * 2;
 
 while( gold - g > conv),
   for i=1:n,
       Ai = A;
       for j=1:m,
           Ai(i,j) = 0.0;
       end;
       ci = Co(:,i);
       ai = (Ai'*Ai)^(-1) * Ai' * ci;
       for j=1:m,
           A (i,j) = ai(j);
       end;
   end;
   gold = g;
   g = trace((Co - A*A' - diag(diag(A*A')))'*(Co - A*A' - diag(diag(A*A'))));
 end;

 %return;
 
 Re = (A*A') + eye(size(C)) - diag(diag(A*A'));
 Um = (N-1) * ( log(det(Re)) - log(det(C)));
 if (Um < 0),
     Um = 0;
 end;

return;
