function varargout = number_nodes(varargin)

if nargin == 0  % LAUNCH GUI

    data=getappdata(0,'data');

    %crear figura
    finestra.N2=350;
    finestra.M2=200;
    pantalla=get(0,'ScreenSize');
    finestra.coordenades=[(pantalla(3)/2)-(finestra.N2/2)  (pantalla(4)/2)-(finestra.M2/2) finestra.N2 finestra.M2];
    handles.graded_nodes = figure('Units','Pixels',...
    'color' , [0.941 0.941 0.941],...
    'Position',finestra.coordenades);
    set(handles.graded_nodes, 'MenuBar','none','Name','Psychological Test Toolbox','Toolbar','none','NumberTitle','off','resize','off');

    %NUMBER OF NODES
    handles.nodes_text1= uicontrol(handles.graded_nodes,'Style','text',...
        'Units','pixels',...
        'position',[60 165 250 25],...
        'FontSize',14,...
        'fontname',char(data.font),...
        'ForegroundColor',[.2 .2 .2],...
        'HorizontalAlignment','left',...
        'FontWeight','normal',...
        'BackgroundColor', [.941 .941 .941],...
        'String','Number of nodes:');

    % EDITABLE BOX OF NUMBER OF NODES
    handles.number_of_nodes= uicontrol(handles.graded_nodes,'Style','edit',...
        'Units','pixels',...
        'position',[230 165 60 25],...
        'FontSize',14,...
        'fontname',char(data.font),...
        'TooltipString','Number of nodes',...
        'max',0,...  
        'BackgroundColor',[1 1 1],...
        'ForegroundColor',[0 0 0],...  
        'Enable','on',...
        'HorizontalAlignment','right',...
        'BackgroundColor', [.941 .941 .941],...
        'String','7');

    str='By incrementing the number of nodes, the EAP estimation will be more precise. However, this could heavily impact the computation time of the EAP scores, so we recommend to use a higher number of nodes (from 20 to 40) only when the precision in the factor scores is mandatory.';

    %TEXT WARNING
    handles.nodes_text2= uicontrol(handles.graded_nodes,'Style','text',...
        'Units','pixels',...
        'position',[10 15 340 140],...
        'FontSize',11,...
        'fontname',char(data.font),...
        'ForegroundColor',[.2 .2 .2],...
        'HorizontalAlignment','left',...
        'FontWeight','normal',...
        'BackgroundColor', [.941 .941 .941],...
        'String',str);

    %OK NUMBER OF NODES SELECTION
    handles.Ok_number_nodes = uicontrol(handles.graded_nodes,...
        'String','OK',...
        'Style','pushbutton',...
        'FontSize',13,...
        'fontname',char(data.font),...  
        'enable','on',...
        'position',[240 10 100 35],...
        'BackgroundColor', [.831 .816 .784]);

    %CANCEL NUMBER OF NODES SELECTION
    handles.Cancel_number_nodes = uicontrol(handles.graded_nodes,...
        'String','Cancel',...
        'Style','pushbutton',...
        'FontSize',13,...
        'fontname',char(data.font),...  
        'enable','on',...
        'position',[130 10 100 35],...
        'BackgroundColor', [.831 .816 .784]);

    set(handles.Cancel_number_nodes,...
        'Callback',{@Cancel_nodes_Callback, handles});
    set(handles.Ok_number_nodes,...
        'Callback',{@Ok_nodes_Callback, handles});
            
    if nargout > 0
            varargout{1} = handles.graded_nodes;
    end


elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end

end


function Cancel_nodes_Callback (gcf,event_data,handles,varargin)

    %% Delete figure and select lineal model
    data=getappdata(0,'data');
    
    data.graded=0;
    
    setappdata(0,'data',data);
    
    delete(gcbf);
    

return;

function Ok_nodes_Callback (gcf,event_data,handles,varargin)

    %% Delete figure and select lineal model
    data=getappdata(0,'data');
    
    n_nodes=str2num(get(handles.number_of_nodes,'string'));
    
    if isempty(n_nodes)
        errordlg('Please, select a valid number of nodes','Psychological Test Toolbox: Error','Error');
        set(handles.number_of_nodes,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif (n_nodes<7 || n_nodes>40)
        errordlg('The number of nodes should be between 7 and 40','Psychological Test Toolbox: Error','Error');
        set(handles.number_of_nodes,'backgroundcolor',[.8 .2 .2]);
        return;
    end;
    
    
    data.graded=1;
    data.n_nodes=n_nodes;
    
    setappdata(0,'data',data);
    
    delete(gcbf);
    

return;