%% REAPGRAD : EAP estimation for a r-dimensional UVA factor model. Joint estimation for all respondents
% see "eapgrad.m" for details              
%               
% ARGUMENTS   : X     ->  MATRIX OF ITEM SCORES (SCORED 1 TO 5)
%               LAM   ->  LOADING FACTOR PATTERN (dimension factors x items)
%               THRES ->  TRHRESHOLD MATRIX (dimension thresholds x items with a final row of zeros)
%               sigj  ->  VECTOR OF ITEM RESIDUAL STANDARD DEVIATIONS  
%
% RETURN VALUE: th    -> factor scores.
%               th_li -> factor scores inferior limit.
%               th_ls -> factor scores upper limit.
%               se    -> standard errors
%               reli  -> reliability of factor scores
%
% DEPENDENCIES: Nseriesk.m
%               Time2String.m
%               creanodos.m
%               eapgrad.m
%               ordnor.m
%               pgrad.m
%

function [th,th_li,th_ls,se,reli] = reapgrad(X, LAM, THRES, sigj,disp);

[m,r]=size(LAM);

%CAL QUE EL VALOR MS BAIX SIGUI 1
X=X+(1-min(min(X)'));

%to create nodos
switch(r)
case {1, 2}
    nodos=creanodos(-4,4,20);
case 3
    nodos=creanodos(-4,4,15);
case 4
    nodos=creanodos(-4,4,10);
otherwise
    nodos=creanodos(-4,4,6);
end;

nodos=creanodos(-4,4,10);
[ni,tmp]=size(nodos);
index_nodos = Nseriesk(ni,r);

%%%%%%%%%%%%%%%%%%%%%%
[n,m] = size(X);


if disp == 1,
   tstart = tic;
   tmp3 = 0; bar1 = waitbar(tmp3,'EAP latent traits estimates');
end;    

th = [];
se=[];
reli=[];
for i=1:n,
   
    [i X(i,:)];
    [th_i,se_i,reli_i] =  eapgrad(X(i,:),LAM, THRES, sigj, nodos, index_nodos);
    th = [th; th_i'];
    se = [se; se_i'];
    reli = [reli; reli_i'];
%    FAC = [FAC; th',se',reli'];
    if (disp >0),
        compT = toc(tstart);
        compT = compT*(n-i)/i;
        s_tmp=['EAP latent traits estimates (Remaining time = ' Time2String( compT ) ')'];
        waitbar(i/n,bar1,s_tmp);
     end;

end;

if disp == 1,close(bar1);end;

%%TH STANDARITZATS!!!
 md=mean(th);
 sx=std(th);
 th=(th-ones(n,1)*md) ./ ( ones(n,1)*sx);

% 95% approximate confidence interval
  th_li= zeros(n,r);
  th_ls= zeros(n,r);
  for i=1:n,
	  for j=1:r
		  th_li(i,j) = th(i,j) - 1.96*se(i,j);
		  th_ls(i,j) = th(i,j) + 1.96*se(i,j);
      end;
  end;
return;
