%% TARROTOB : Semi-specified oblique rotation
%
% ARGUMENTS   : F    -> Factor soultion to be rotated
%               W    -> Target binary matrix
%               conv -> Convergence index
%
% RETURN VALUE: A    -> Factor solution rotated (pattern)
%               T    -> Transformation matrix
%               PHI  -> Inter-factors correlation
%               
function [A, T, PHI, ST ] = tarrotob (F, W, conv);

 [m,r] = size(F);
 TA  = eye(r);
 PHI = eye(r);
 A   = F * TA;

 f1 = 0;
 for g=1:m,
   for h=1:r,
     f1 = f1 + (A(g,h)-W(g,h)*A(g,h))^2;
   end;
 end;

 f1old = f1;

 iter = 1;
 again = 1;
 while again ==1,
    for g=1:r,
        for h=1:r,
            if (g~=h),
                agg=0;bgg=0;bgh=0;
                for j=1:m,
                    agg = agg + ((1-W(j,g)) * A(j,g)^2);
                    bgg = bgg + ((1-W(j,h)) * A(j,g)^2);
                    bgh = bgh + ((1-W(j,h)) * A(j,g) * A(j,h));
                end;

                delta = (bgh-PHI(g,h)*agg)/(agg+bgg);
                gamma = sqrt(delta^2 + 2*PHI(g,h)*delta+1 );

                for j=1:m,
                    A(j,h) = (-1.0 * delta * A(j,g))+ A(j,h);
                    A(j,g) = gamma * A(j,g);
                end;

                for j=1:r,
                    TA(j,h) = (-1.0 * delta * TA(j,g))+ TA(j,h);
                    TA(j,g) = gamma * TA(j,g);
                end;

                for j=1:r,
                    PHI(g,j) = (PHI(g,j)+ delta*PHI(h,j))/gamma;
                end;
                PHI(g,g) = 1;
                for j=1:r,
                    PHI(j,g) = PHI(g,j);
                end;

            end;
        end;
    end;

    f1old = f1;

    f1 = 0;
    for g=1:m,
      for h=1:r,
          f1 = f1 + (A(g,h)-W(g,h)*A(g,h))^2;
      end;
    end;

    if f1 < .1*conv,
       again = 0;
    end;

    if abs (f1 - f1old) < conv,
       again = 0;
    end;

    if iter > 50,
       again = 0;
    end;

    iter = iter + 1;

 end;

 ST = A * PHI;
 T = TA;
return
